/*
 * Copyright 2008 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import org.codehaus.groovy.grails.commons.*;
import jp.grails.plugins.email.artefact.*;
import org.codehaus.groovy.grails.web.plugins.support.WebMetaUtils;
import org.codehaus.groovy.grails.web.pages.GroovyPage;

/**
 * 
 * @author Tsuyoshi Yamamoto
 */
class MailSenderGrailsPlugin {
  def version = 0.1
  def author = "Tsuyoshi Yamamoto"
  def authorEmail = "tyama@xmldo.jp"
  def title = "Mail Sender Plugin"
  def description = '''Simple e-mail sender plugin
  '''
  def documentation =""

  def dependsOn = [controllers:grails.util.GrailsUtil.getGrailsVersion()]
  def observe = ['controllers']
  def loadAfter=['controllers']
  def watchedResources = ["file:./grails-app/controllers/**/*Controller.groovy",
          "file:./plugins/*/grails-app/controllers/**/*Controller.groovy",
          "file:./grails-app/emails/*Email.groovy",
          "file:./plugins/*/grails-app/emails/*Email.groovy"]

  def artefacts = [EmailArtefactHandler]

  def doWithSpring = {
    mailSender(org.springframework.mail.javamail.JavaMailSenderImpl) {
      host = '127.0.0.1'
    }
    simpleMailMessage(org.springframework.mail.SimpleMailMessage) {
      from = 'sender@yourdomainname.jp'
    }

    def mailFrom = application.config.mail?.smtp?.from
    abstractMailMessage{ 
      from = mailFrom?:'sender@yourdomainname.jp'
    }

    for (em in application.emailClasses) {
      "send${em.shortName}"(org.springframework.mail.SimpleMailMessage){bean->
        bean.parent = abstractMailMessage
        if(em.getPropertyValue("subject")) subject=em.getPropertyValue("subject")
        if(em.getPropertyValue("from")) from=em.getPropertyValue("from")
        if(em.getPropertyValue("to")) to=em.getPropertyValue("to")
        if(em.getPropertyValue("cc")) cc=em.getPropertyValue("cc")
        if(em.getPropertyValue("bcc")) bcc=em.getPropertyValue("bcc")
        if(em.getPropertyValue("replyTo")) replyTo=em.getPropertyValue("replyTo")
        if(em.getPropertyValue("sentDate")) sentDate=em.getPropertyValue("sentDate")
      }
    }
  }
 
  def doWithApplicationContext = { applicationContext ->
    def ms = applicationContext.mailSender
    def smtpHost = application.config.mail?.smtp?.host
    if(smtpHost) ms.host=smtpHost
    
    def smm = applicationContext.simpleMailMessage
    def mailFrom = application.config.mail?.smtp?.from
    if(mailFrom) smm.from=mailFrom
  }

  def doWithWebDescriptor = { xml ->
  }
                                      
  def doWithDynamicMethods = { ctx ->
    application.controllerClasses.each { controllerClass -> 
      def mc = controllerClass.metaClass
      registerControllerProps(mc,ctx)
    }
  }

  def onChange = { event ->
    if (application.isArtefactOfType(ControllerArtefactHandler.TYPE, event.source)) {
      def controllerClass = application.addArtefact(ControllerArtefactHandler.TYPE, event.source)
      MetaClass mc = controllerClass.metaClass
      registerControllerProps(mc,event.ctx)
    }
    if (application.isArtefactOfType(EmailArtefactHandler.TYPE, event.source)) {
      def em = application.addArtefact(EmailArtefactHandler.TYPE, event.source)
      def beans = beans{
        "send${em.shortName}"(org.springframework.mail.SimpleMailMessage){bean->
          bean.parent = ref("abstractMailMessage")
          if(em.getPropertyValue("subject")) subject=em.getPropertyValue("subject")
          if(em.getPropertyValue("from")) from=em.getPropertyValue("from")
          if(em.getPropertyValue("to")) to=em.getPropertyValue("to")
          if(em.getPropertyValue("cc")) cc=em.getPropertyValue("cc")
          if(em.getPropertyValue("bcc")) bcc=em.getPropertyValue("bcc")
          if(em.getPropertyValue("replyTo")) replyTo=em.getPropertyValue("replyTo")
          if(em.getPropertyValue("sentDate")) sentDate=em.getPropertyValue("sentDate")
        }
      }
      if(event.ctx) {
        beans.registerBeans(event.ctx)
      }
    }
  }
  
  /**
   * Add method: sendEmail(from:"",to:[""],subject:"",text:"")
   */
  def registerControllerProps(mc,ctx){
    
    mc.sendEmail = {params->
      def message = new org.springframework.mail.SimpleMailMessage(ctx.simpleMailMessage)
      def encoding = params.encoding?params.remove("encoding"):"utf-8"
      params.each{k,v-> message."$k"=v }
      try {
        def sender =ctx.mailSender
        sender.defaultEncoding=encoding
        sender.send(message)
      } catch (Exception e) {
        e.printStackTrace()
      }finally {
        return ""
      }
    }
    
    mc.methodMissing = {String methodName, args ->
      if(ctx."$methodName"){
        def params = args[0]
        def encoding = params.encoding?params.remove("encoding"):"utf-8"
        def fillin = params.fillin?params.remove("fillin"):[:]
        def message = new org.springframework.mail.SimpleMailMessage(ctx."$methodName")
        params.each{k,v->
          message."$k"=v
        }
        def emailArtefact = ctx.grailsApplication.getArtefact(EmailArtefactHandler.TYPE, (methodName-"send"))
        message.text = emailArtefact.getText(fillin)
        if(emailArtefact.getPropertyValue("encoding")){
          encoding=emailArtefact.getPropertyValue("encoding")
        }
        try {
          def sender =ctx.mailSender
          sender.defaultEncoding=encoding
          sender.send(message)
        } catch (Exception e) {
          e.printStackTrace()
        }finally {
          return ""
        }
      }else{
        args = args == null ? [] as Object[] : args
        def tagName = "${GroovyPage.DEFAULT_NAMESPACE}:$methodName"
        GrailsClass tagLibraryClass = ctx.grailsApplication.getArtefactForFeature(
                TagLibArtefactHandler.TYPE, tagName.toString())
        def result
        if (tagLibraryClass) {
            synchronized (mc) {
                WebMetaUtils.registerMethodMissingForTags(mc, ctx, tagLibraryClass, methodName)
            }
            result = mc.invokeMethod(delegate, methodName, args)
        }else {
            throw new MissingMethodException(methodName, delegate.class, args)
        }
        return result
      }
    }
  } 
                                                                         
  def onApplicationChange = { event -> }
}
